/* im2avi_t.h copyright 2001, 2002 by Charl P. Botha http://cpbotha.net/
 *
 * $Id: im2avi_t.h,v 1.6 2003/06/20 12:28:53 cpbotha Exp $
 *
 * im2avi: frontend to avifile and Magick++ for making AVIs from images.
 * Copyright (C) 2001,2002  Charl P. Botha
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _im2avi_h
#define _im2avi_h

class Fl_Button;
class Fl_File_Chooser;
class Fl_Value_Input;
class main_window_t;

#include <avm_default.h> // from avifile, required for fourcc_t;
#include <vector>

/**
 * im2avi application class.  This contains most of the current functionality,
 * but will be used later as communications hub if more modules are
 * added.  If you instantiate one of these, you have an encoding application.
 * @author Charl P. Botha <cpbotha@ieee.org>
 */
class im2avi_t
{
protected:
   /**
    * Class created from fluid data file during compilation.  This contains all
    * the ui elements.
    */
   main_window_t* main_window;
   /**
    * This is the file chooser that's used to add images to the images list.
    * We have this pointer so we can instantiate once and destroy once,
    * thus also remaining stateful between calls to the add_images_fs()
    * method.
    */
   Fl_File_Chooser* file_chooser;
   /**
    * A list of fourccs corresponding to the codec descriptions in the
    * codec Fl_Choice.
    */
   vector<fourcc_t> avi_codec_fourccs;
   /**
    * All input images will be resized to this width.  Kept synced to the
    * ui with update_image_width().
    * @see update_image_width()
    */
   int image_width;
   /**
    * All input images will be resized to this height.  Kept synced to the
    * ui with update_image_height().
    * @see update_image_height()
    */
   int image_height;
   /**
    * We try and maintain this aspect ratio when the user changes either
    * the width or the height.
    */
   float image_aspect_ratio;
public:
   /**
    * The constructor instantiates main_window and sets up all pertinent
    * ui elements.
    */
   im2avi_t(int argc, char* argv[]);
   /**
    * Well, doh, things get de-initialised.
    */
   ~im2avi_t();
   /**
    * This thunks through to Fl::run() and returns the result.
    */
   int run(void);
   /**
    * The global mw_button_cb calls this method, which looks at user_data
    * and then executes the applicable method.
    */
   void mw_button_cb(Fl_Button* w, void* user_data);
   /**
    * The global mw_input_cb calls this method, which looks at user_data
    * and then executes the applicable method.
    */
   void mw_input_cb(Fl_Value_Input* w, void* user_data);
   /**
    * Shows file selection dialog and allows the user to select multiple files.
    * The files are added to the end of the images list.
    */
   void add_images_fs(void);
   /**
    * All selected images are removed.
    */
   void remove_images(void);
   /**
    * A list with fully qualified filenames is loaded and all the filenames are
    * put in the images list.  All previous content of the images_list is
    * removed.  This list can be created with im2avi or by other means.
    */
   void load_images_list(void);
   /**
    * The contents of the images list is written to a text file.  This file can be
    * loaded again by im2avi.
    */
   void save_images_list(void);
   /**
    * Moves the whole images list selection to index "to", 1-based.
    */
   void move_selection(int to);
   /**
    * Moves the whole images list selection differentially, i.e. current
    * position + diff.
    */
   void move_selection_diff(int diff);
   /**
    * Move whole images list selection so that it is at the end of the list.
    */
   void move_selection_to_bottom(void);
   /**
    * Update the image_width variable according to the image width
    * ui element.  In addition, update the height according to the
    * current aspect ratio if the constrain aspect check box is set.
    * @param ignore_aspect If true, ignore the aspect ratio, whether
    * the constrain aspect ratio check box is set or not.
    */
   void update_image_width(bool ignore_aspect = false);
   /**
    * Update the image_height variable according to the image height
    * ui element.  In addition, update the width according to the
    * current aspect ratio if the constrain aspect check box is set.
    * @param ignore_aspect If true, ignore the aspect ratio, whether
    * the constrain aspect ratio check box is set or not.
    */
   void update_image_height(bool ignore_aspect = false);
   /**
    * Set the class variable aspect_ratio and also update the UI element.
    */
   void update_image_aspect_ratio(float new_aspect);
   /**
    * Pop up a file selection dialogue so that the user can enter a filename
    * for the output AVI.
    */
   void browse_avi_filename(void);
   /**
    * Determine the width and height of the first selected image and set
    * the UI and internal width and height variables accordingly.
    */
   void auto_wxh(void);
   /**
    * Create an AVI file (and stream) according to all configured parameters.
    */
   void make_avi(void);
   /**
    * Update the progress bar and label (all ui).
    * @param percentage Current progress as a percentage.
    */
   void update_progress_ui(float percentage);
};

#endif

